<?php

/**
 * Data model for recipes. Handles retrieving recipes based on various criteria.
 * 
 * @package Models
 *
 */
class Recipe_Model extends AbstractModel
{
	/**
	 * Retrieves a list of all recipes that match $query (uses recipe name to compare).
	 * 
	 * @param string $query The query with which to find matches.
	 * @return array
	 */
	function getRecipesByName($query)
	{
		$query = $this->db->getLike($query);
		$recipes = $this->db->queryAll('select id, name, directions from recipe where name LIKE ?', array($query));

		$recipes = $this->_joinIngredients($recipes);

		return $recipes;
	}

	/**
	 * Retrieves a list of recipes that match $query (uses recipe ingredients to compare).
	 * 
	 * @param string $query The query with which to find matches.
	 * @return array
	 */
	function getRecipesByIngredient($query)
	{
		$query = $this->db->getLike($query);

		// get recipes with ingredients matching query.
		$recipes = $this->db->queryAll('
			select distinct r.id, r.name, r.directions
			from recipe r, recipeingredient ri, ingredient i
			where r.id = ri.recipeId and ri.ingredientId = i.id
				and i.name like ?', array($query));

		$recipes = $this->_joinIngredients($recipes);

		return $recipes;
	}

	/**
	 * Retrieves a recipe using its id.
	 * 
	 * @param int $id The Recipe Id
	 * @return array An associate array with all the recipe data.
	 * 
	 */
	function getRecipeById($id)
	{
		$recipe = $this->db->queryRow('
			select id, name, directions
			from recipe
			where id = ?', array($id));
		$recipe['ingredients'] = $this->_getIngredients($id);

		return $recipe;
	}

	/**
	 * Retrieves a list of recipes which match the given special id.
	 * 
	 * @param int $specialId The special id for which to get associated recipes.
	 * @return array
	 */
	function getRecipesForSpecial($specialId)
	{
		// get recipes with ingredients mentioned in special
		$recipes = $this->db->queryAll('
			select distinct r.id, r.name, r.directions
			from recipe r, specialingredient si, recipeingredient ri
			where si.ingredientId = ri.ingredientId and ri.recipeId = r.id
				and si.specialId = ?', array($specialId));

		$recipes = $this->_joinIngredients($recipes);
		return $recipes;
	}

	/**
	 * @ignore
	 */
	function _joinIngredients($recipes) {
		$result = array();
		foreach ($recipes as $recipe) {
			$recipe['ingredients'] = $this->_getIngredients($recipe['id']);
			$result[] = $recipe;
		}

		return $result;
	}

	/**
	 * @ignore
	 */
	function _getIngredients($recipeId) {
		$ingredients = $this->db->queryAll('
			select ri.recipeId, ri.ingredientId, i.name, ri.quantity, ri.preparation
			from recipeingredient ri, ingredient i
			where ri.recipeId = ? and ri.ingredientId = i.id', array($recipeId));

		return $ingredients;
	}
}

?>