<?php

/**
 * Data model for specials. Handles retrieving specials based on various criteria.
 * 
 * @package Models
 *
 */
class Special_Model extends AbstractModel
{
	/**
	 * Given a list of specials and zip code, filters the specials down to specials
	 * which are available in the given zip code.
	 * 
	 * @param array $specials The list of specials to filter.
	 * @param string $zipCode The zip code to use when filtering specials. 
	 *                        Must be a 5-digit string with numeric digits only.
	 */
	function filterByZipCode($specials, $zipCode)
	{
		if (!$zipCode) {
			return $specials;
		}

		$zipCode = intval(preg_replace('([^0-9])', '', $zipCode));
		$result = array();
		foreach ($specials as $special) {
			if (intval(preg_replace('([^0-9])', '', $special['store']['zipCode'])) == $zipCode) {
				$result[] = $special;
			}
		}

		return $result;
	}

	/**
	 * Retrieves a complete list of zip codes supported by the database.
	 */
	function getAllZipCodes()
	{
		return $this->db->queryColumn('select distinct zipCode from storelocation');
	}

	/**
	 * Retrieves a list of specials which match $query by name.
	 * 
	 * @param string $query The query to match with.
	 * @return array
	 */
	function getSpecialsByName($query)
	{
		$query = $this->db->getLike($query);

		$specials = $this->db->queryAll('
			select s.id, s.title, s.description, s.price
			from special s
			where
				(s.title like ? or s.description like ?)',
			array($query, $query));

		$specials = $this->_joinStoreInfo($specials);
		return $specials;
	}

	/**
	 * Retrieves a list of specials which match $query by the ingredient(s) associated with the special.
	 * 
	 * @param string $query The query to match with.
	 * @return array
	 */
	function getSpecialsByIngredient($query)
	{
		$query = $this->db->getLike($query);

		$specials = $this->db->queryAll('
			select s.id, s.title, s.description, s.price
			from special s, specialingredient si, ingredient i
			where si.specialId = s.id
				and i.id = si.ingredientId
				and i.name LIKE ?', array($query));

		$specials = $this->_joinStoreInfo($specials);
		return $specials;
	}

	/**
	 * Retrieves a list of specials available in the given zip code.
	 * 
	 * @param string $zipCode The zip code to match with.
	 * @return array
	 */
	function getSpecialsByArea($zipCode)
	{
		$zipCode = $this->db->getLike($zipCode);
		$specials = $this->db->queryAll('
			select s.id, s.title, s.description, s.price
			from special s, storeLocation sl
			where s.storeLocationId = sl.id
				and sl.zipCode like ?', array($zipCode));
		$specials = $this->_joinStoreInfo($specials);
		return $specials;
	}

	/**
	 * Retrieves a single special by its id.
	 * 
	 * @param int $id The special id.
	 * @return array An associate array with all the special's data.
	 */
	function getSpecialById($id)
	{
		$special = $this->db->queryRow('
			select id, title, description, price
			from special
			where id = ?', array($id));

		if ($special) {
			$special['store'] = $this->_getStoreInfo($special);
			return $special;
		}

		return null;
	}

	/**
	 * Retrieves a list of associate
	 * 
	 * @param int $recipeId The recipe id for which to return associated specials.
	 * @param string $zipCode [Optional] The zip code in which to search for associated specials.
	 * @return array
	 */
	function getSpecialsForRecipe($recipeId, $zipCode = null)
	{
		if ($zipCode) {
			$specials = $this->db->queryAll('
				select distinct s.id, s.title, s.description, s.price
				from special s, specialingredient si, recipeingredient ri, storelocation sl
				where ri.ingredientId = si.ingredientId
					and s.id = si.specialId
					and s.storeLocationId = sl.id
					and ri.recipeId = ?
					and sl.zipCode like ?', array($recipeId, $zipCode));
		} else {
			$specials = $this->db->queryAll('
				select distinct s.id, s.title, s.description, s.price
				from special s, specialingredient si, recipeingredient ri
				where ri.ingredientId = si.ingredientId and s.id = si.specialId
					and ri.recipeId = ?', array($recipeId));
		}

		$specials = $this->_joinStoreInfo($specials);
		return $specials;
	}

	/**
	 * @ignore
	 */
	function _joinStoreInfo($specials) {
		$result = array();
		foreach ($specials as $special) {
			$special['store'] = $this->_getStoreInfo($special);
			$result[] = $special;
		}

		return $result;
	}

	/**
	 * @ignore
	 */
	function _getStoreInfo($special) {
		return $this->db->queryRow('
			select sl.id as storeLocationId, sl.storeChainId, sl.address, sl.city, sl.state, sl.zipCode, sl.phone,
				sc.name, sc.websiteUrl
			from special s, storelocation sl, storechain sc
			where s.storeLocationId = sl.id and sl.storeChainId = sc.id
				and s.id = ?', array($special['id']));
	}
}

?>