<?php

/**
 * Data model for users. Handles all data-related user functions, and 
 * also handles login, logout, and registration.
 * 
 * @package Models
 *
 */
class User_Model extends AbstractModel {

    /**#@+
     * @access private
	 * @ignore
     */
	private $logged_in;
	private $id;
	private $username;
	private $password;
	private $email;
	private $zipCode;
    /**#@-*/

	/**
	 * @ignore
	 */
	function __construct()
	{
		parent::__construct();
		if (!session_id())
			session_start();

		$this->initializeLogin();
		$this->initializeFromDb();
	}

	/**
	 * Populate data with user login information from sessions and/or cookies.
	 * @private
	 */
	private function initializeLogin()
	{
		if (isset($_SESSION['user_id'])) {
			$this->logged_in = true;
			$this->id = intval($_SESSION['user_id']);
		} else {
			$this->logged_in = false;
		}
	}

	/**
	 * Using the user's login information, populate user model with data.
	 * @private
	 */
	private function initializeFromDb()
	{
		if ($this->isLoggedIn()) {
			$userdata = $this->db->queryRow(
				'select id, username, password, email, zipCode
				from user
				where id = ?',
				array($this->id));
			if (!$userdata) {
				// todo: error handling on invalid login.
				// todo: check password against database (to prevent user changing the username in their cookie).
				trigger_error('Error initializing user information from database.');
				return;
			}

			$this->username = $userdata['username'];
			$this->password = $userdata['password'];
			$this->email = $userdata['email'];
			$this->zipCode = $userdata['zipCode'];
		} else {
			$this->id = -1;
			$this->username = 'Anonymous';
			$this->password = '';
			$this->email = '';
			if (isset($_SESSION['zipCode']))
				$this->zipCode = ($_SESSION['zipCode']);
			else if (isset($_REQUEST['zipCode']))
				$this->zipCode = ($_REQUEST['zipCode']);
			else
				$this->zipCode = '';
		}
	}

	/**
	 * Logs in the user given username and password.
	 * 
	 * @param string $username The username.
	 * @param string $password The md5'd password.
	 * @return bool True if login successful, False otherwise.
	 */
	function login($username, $password)
	{
		Session_Handler::start();

		// check that the login credentials are valid.
		$result = $this->db->queryRow(
			'select id, password
			from user
			where username = ?', array($username));
		if (!$result || $result['password'] != $password){
			Session_Handler::addError('Invalid username or password.', 'login');
			return false;
		}

		$this->id = $result['id'];

		// login using session.
		$_SESSION['user_id'] = $this->id;
		$this->logged_in = true;

		return true;
	}

	/**
	 * Logs out the currently logged in user.
	 * Does nothing if the user is already logged out.
	 * 
	 */
	function logout()
	{
		Session_Handler::end();
		$this->logged_in = false;
	}

	/**
	 * Registers a new user using the given information.
	 * 
	 * @param string $username The desired username.
	 * @param string $password The desired password.
	 * @return bool True if registration successful, False otherwise.
	 */
	function register($username, $password)
	{
		if (!$this->_validateUsername($username)) {
			return false;
		}
		
		if (!$this->_validatePassword($password)) {
			return false;
		}

		// check that username is valid
		$this->db->query(
			'insert into
				user (username, password)
				values (?, ?)',
				array($username, md5($password)));
		if (!$this->db->lastInsertId()) {
			Session_Handler::addError('Error creating account. That username already taken.', 'login');
			return false;
		}
		
		return true;
	}

	/**
	 * Gets whether the user is logged in.
	 * @return bool True if user is logged in, False if user is an anonymous user.
	 */
	function isLoggedIn()
	{
		return $this->logged_in;
	}

	/**
	 * Gets the user's id.
	 * 
	 */
	function id()
	{
		if (!$this->isLoggedIn()) {
			// todo: error if accessing id without being logged in
			trigger_error('Accessing ID without being logged in.');
		}

		return $this->id;
	}

	/**
	 * Gets the user's username.
	 * 
	 */
	function username()
	{
		return $this->username;
	}

	/**
	 * Gets (or sets) the user's zip code.
	 * 
	 */
	function zipCode($set = null) {
		if ($set) {
			$this->zipCode = $set;

			if ($this->isLoggedIn() && is_int($this->zipCode)) {
				$this->db->query('update user set zipCode = ? where id = ?', array($set, $this->id()));
			}
		}

		if ($this->zipCode) {
			return $this->zipCode;
		} else {
			if (Url::get('zipCode')) {
				return Url::get('zipCode');
			}
		}

		// return null as default
		return null;
	}

	/**
	 * Gets (or sets) the user's password.
	 * 
	 * @param string $newPassword The new password to use. Must not be md5'd yet.
	 * @return bool True iff successful.
	 */
	function password($newPassword) {
		if ($this->_validatePassword($newPassword) && $this->isLoggedIn()) {
			$this->db->query('update user set password = ? where id = ?', array(md5($newPassword), $this->id()));
		}

		// return true if successful, false otherwise.
		return true;
	}

	/**
	 * @ignore
	 */
	function _validateUsername($username)
	{
		if (!$username) {
			Session_Handler::addError('Username wasn\'t specified.', 'login');
			return false;
		}

		if (trim($username) != $username) {
			Session_Handler::addError('Username has spaces to the right or left.', 'login');
			return false;
		}
		
		if (strlen($username) < 2 || strlen($username) > 70) {
			Session_Handler::addError('Username must be at least 2 letters long, and no longer than 70 characters.', 'login');
			return false;
		}
		
		if (!preg_match('/([a-zA-Z0-9_])*/', $username)) {
			Session_Handler::addError('Username can only contain letters, numbers, dashes, and underscores.', 'login');
			return false;
		}

		return true;
	}

	/**
	 * @ignore
	 */
	function _validatePassword($password)
	{
		//
		// validate password
		// (7 <= length <= 20)
		// contains combination of [a-zA-Z0-9!@#$%^&*()-_+=]
		$safe = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!@#$%^&*()-_+=";

		//
		// update password in database.
		if (strlen($newPassword) < 21 && strlen($newPassword) > 6 && str_replace($safe, "", $newPassword) == "") {
			Session_Handler::addError('Password must have letters, numbers, or the symbols on your keyboard. Must be more than 6 characters long and less than 21.', 'login');
			return false;
		}
		
		return true;
	}

	/**
	 * Gets an associate array. Designed for easy passing data to template.
	 * 
	 */
	function getArray()
	{
		return array(
			'id' => $this->id,
			'username' => $this->username,
			'password' => $this->password,
			'email' => $this->email,
			'zipCode' => $this->zipCode,
			'isLoggedIn' => $this->isLoggedIn(),
		);
	}
}

?>